@@ -3,6 +3,7 @@ from django.db.models import Q, Max, Case, Value, When, Exists, OuterRef, \
     UniqueConstraint, Subquery
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
+from django.core.cache import cache
 from django.core.exceptions import FieldError, ValidationError
 from django.template import Template, loader
 from django.utils.functional import cached_property
@@ -13,8 +14,6 @@ from django.utils import timezone
 from django.urls import reverse
 from polymorphic.models import PolymorphicModel
 import uuid
-import markdown
-from markdown_link_attr_modifier import LinkAttrModifierExtension
 from itertools import groupby
 from pathlib import Path
 from datetime import timedelta
@@ -23,11 +22,14 @@ import os
 from ..stock import StockWidget
 from ..filetype import FileType
 from ..utils import create_model, remove_p, send_email, submission_link, \
-    thumbnail_path
+    thumbnail_path, MarkdownFormatter
 from .ranked import RankedModel, UnderscoredRankedModel
 from .automatic import AutoSlugModel
 
 
+markdown = MarkdownFormatter()
+
+
 class Program(AutoSlugModel):
     class Meta:
         ordering = ['created']
@@ -53,12 +55,6 @@ class Program(AutoSlugModel):
             msg = 'Identifier (with hyphens removed) must be unique.'
             raise ValidationError(msg)
     
-    @cached_property
-    def markdown(self):
-        return markdown.Markdown(extensions=[
-            LinkAttrModifierExtension(new_tab='external_only')
-        ])
-    
     def visible_forms(self):
         pub = self.forms.exclude(status=Form.Status.DRAFT)
         return pub.exclude(options__hidden__isnull=False)
@@ -173,6 +169,11 @@ class Form(AutoSlugModel):
         return create_model(name, fields, program=self.program.db_slug,
                             base_class=SubmissionItem, meta=Meta)
     
+    def cache_dirty(self):
+        version = cache.get('models_version')
+        if version is None: cache.set('models_version', 1, timeout=None)
+        else: cache.incr('models_version')
+    
     def publish_model(self, model, admin=None):
         with connection.schema_editor() as editor:
             editor.create_model(model)
@@ -181,22 +182,17 @@ class Form(AutoSlugModel):
         ctype.save()
         ContentType.objects.clear_cache()
         
-        from ..admin import site
-        site.register(model, admin)
+        self.cache_dirty()
     
     def unpublish_model(self, model):
-        from ..admin import site
-        match = None
-        for m in site._registry.keys():
-            if m._meta.db_table == model._meta.db_table: match = m
-        if match: site.unregister(match)
+        self.cache_dirty()
         
         ctype = ContentType.objects.get_for_model(model)
         ctype.delete()
         ContentType.objects.clear_cache()
         with connection.schema_editor() as editor:
             editor.delete_model(model)
-            
+    
     def publish(self):
         if self.status != self.Status.DRAFT: return
         
@@ -348,21 +344,15 @@ class Form(AutoSlugModel):
             return True
         return False
     
-    def markdown(self, text):
-        md = self.program.markdown
-        return md.reset().convert(text)
-    
     def review_pre(self, prefix=''):
         name = prefix + 'review_pre'
         if name in self.options:
-            md = self.program.markdown
-            return mark_safe(self.markdown(self.options[name]))
+            return mark_safe(markdown.convert(self.options[name]))
         return ''
     
     def review_post(self):
         if 'review_post' in self.options:
-            md = self.program.markdown
-            return mark_safe(self.markdown(self.options['review_post']))
+            return mark_safe(markdown.convert(self.options['review_post']))
         return ''
     
     def submit_submission(self, submission):
@@ -393,8 +383,7 @@ class Form(AutoSlugModel):
     
     def thanks(self):
         if 'thanks' in self.options:
-            md = self.program.markdown
-            return mark_safe(self.markdown(self.options['thanks']))
+            return mark_safe(markdown.convert(self.options['thanks']))
     
     def emails(self):
         if 'emails' in self.options: return self.options['emails']
@@ -445,7 +434,7 @@ class FormLabel(models.Model):
         return self.path
     
     def display(self, inline=False):
-        s = self.form.program.markdown.reset().convert(self.text)
+        s = markdown.convert(self.text)
         if inline: return mark_safe(remove_p(s))
         return mark_safe(s)
     
